ui_print " "

# info
MODVER=`grep_prop version $MODPATH/module.prop`
MODVERCODE=`grep_prop versionCode $MODPATH/module.prop`
ui_print " ID=$MODID"
ui_print " Version=$MODVER"
ui_print " VersionCode=$MODVERCODE"
ui_print " MagiskVersion=$MAGISK_VER"
ui_print " MagiskVersionCode=$MAGISK_VER_CODE"
ui_print " "

# bit
if [ "$IS64BIT" != true ]; then
  abort "- This module is only for 64 bit architecture."
else
  ui_print "- 64 bit"
  ui_print " "
fi

# sdk
NUM=28
if [ "$API" -lt $NUM ]; then
  ui_print "! Unsupported SDK $API. You have to upgrade your"
  ui_print "  Android version at least SDK API $NUM to use this"
  ui_print "  module."
  abort
else
  ui_print "- SDK $API"
  ui_print " "
fi

# .aml.sh
mv -f $MODPATH/aml.sh $MODPATH/.aml.sh

# mod ui
if getprop | grep -Eq "dolby.mod.ui\]: \[1"; then
  APP=DaxUI
  FILE=/sdcard/$APP.apk
  DES=$MODPATH/system/priv-app/$APP
  ui_print "- Using modified UI apk..."
  if [ -f $FILE ]; then
    cp -f $FILE $DES
    chmod 0644 $DES/$APP.apk
    ui_print "  Applied"
  else
    ui_print "  ! There is no $FILE file."
    ui_print "    Please place the apk to your internal storage first"
    ui_print "    and reflash!"
  fi
  ui_print " "
fi

# cleaning
ui_print "- Cleaning..."
APP="DaxUI
     daxService
     com.dolby.daxappui
     com.dolby.daxservice"
for APPS in $APP; do
 rm -f `find /data/dalvik-cache /data/system/package_cache -type f -name *$APPS*`
done
rm -f /data/vendor/dolby/dax_sqlite3.db
rm -f $MODPATH/LICENSE
rm -rf $MODPATH/unused
conflict() {
  rm -rf /data/adb/modules_update/$MODULE
  rm -f /data/adb/modules/$MODULE/update
  touch /data/adb/modules/$MODULE/remove
  sh /data/adb/modules/$MODULE/uninstall.sh
  rm -f /data/adb/modules/$MODULE/uninstall.sh
  rm -rf /metadata/magisk/$MODULE
  rm -rf /mnt/vendor/persist/magisk/$MODULE
  rm -rf /persist/magisk/$MODULE
  rm -rf /data/unencrypted/magisk/$MODULE
  rm -rf /cache/magisk/$MODULE
}
MODULE=dolbyatmos
conflict
MODULE=DolbyAudio
conflict
MODULE=MotoDolby
conflict
ui_print " "

# store
cleanup() {
  sh $FOLDER/uninstall.sh
  FOLDER=/data/adb/modules_update/$MODID
  sh $FOLDER/uninstall.sh
}

# cleanup
FOLDER=/data/adb/modules/$MODID
FILE=$FOLDER/module.prop
if getprop | grep -Eq "dolby.cleanup\]: \[1"; then
  ui_print "- Cleaning-up $MODID data..."
  cleanup
  ui_print " "
elif [ -d $FOLDER ] && ! grep -Eq "$MODNAME" $FILE; then
  ui_print "- Different version detected"
  ui_print "  Cleaning-up $MODID data..."
  cleanup
  ui_print " "
fi

# store
clean_rules() {
  rm -f $MODPATH/sepolicy.rule
  rm -rf /metadata/magisk/$MODID
  rm -rf /mnt/vendor/persist/magisk/$MODID
  rm -rf /persist/magisk/$MODID
  rm -rf /data/unencrypted/magisk/$MODID
  rm -rf /cache/magisk/$MODID
}
permissive() {
  SELINUX=`getenforce`
  if [ "$SELINUX" == Enforcing ]; then
    setenforce 0
    SELINUX=`getenforce`
    if [ "$SELINUX" == Enforcing ]; then
      abort "! Your device can't be turned to Permissive state."
    fi
    setenforce 1
  fi
  sed -i '1i\
SELINUX=`getenforce`\
if [ "$SELINUX" == Enforcing ]; then\
  setenforce 0\
fi\' $MODPATH/post-fs-data.sh
}
backup() {
  if [ ! -f $FILE.orig ] && [ ! -f $FILE.bak ]; then
    cp -f $FILE $FILE.orig
  fi
}
patch_manifest() {
  if [ -f $FILE ]; then
    backup
    ui_print "- Patching"
    ui_print "  $FILE"
    ui_print "  directly..."
    sed -i '/<manifest/a\
    <hal format="hidl">\
        <name>vendor.dolby.hardware.dms</name>\
        <transport>hwbinder</transport>\
        <version>1.0</version>\
        <interface>\
            <name>IDms</name>\
            <instance>default</instance>\
        </interface>\
        <fqname>@1.0::IDms/default</fqname>\
    </hal>' $FILE
    ui_print " "
  fi
}
patch_hwservice() {
  if [ -f $FILE ]; then
    backup
    ui_print "- Patching"
    ui_print "  $FILE"
    ui_print "  directly..."
    sed -i '1i\
vendor.dolby.hardware.dms::IDms u:object_r:hal_dms_hwservice:s0' $FILE
    ui_print " "
  fi
}
check_patch() {
  if ! grep -Eq "$CHECK" $FILE; then
    ui_print "! Patching failed. Please READ #troubleshootings!"
    abort
  fi
}

# permissive
if getprop | grep -Eq "dolby.permissive\]: \[1"; then
  ui_print "- Using permissive method"
  clean_rules
  permissive
  ui_print " "
elif getprop | grep -Eq "dolby.permissive\]: \[2"; then
  ui_print "- Using both permissive and SE policy patch"
  permissive
  ui_print " "
fi

# magisk
if [ -d /sbin/.magisk ]; then
  MAGISKTMP=/sbin/.magisk
else
  MAGISKTMP=`find /dev -mindepth 2 -maxdepth 2 -type d -name .magisk`
fi

# remount
set_read_write() {
  if [ -d $BLOCK ]; then
    blockdev --setrw $BLOCK/system_ext$SLOT
    blockdev --setrw $BLOCK/vendor$SLOT
    blockdev --setrw $BLOCK/cust$SLOT
    blockdev --setrw $BLOCK/system$SLOT
  fi
}
if [ "$BOOTMODE" == true ]; then
  SYSTEMX=$MAGISKTMP/mirror/system_ext
  VENDOR=$MAGISKTMP/mirror/vendor
  if [ "$SYSTEM_ROOT" == true ]; then
    SYSTEM=$MAGISKTMP/mirror/system_root
  else
    SYSTEM=$MAGISKTMP/mirror/system
  fi
  BLOCK=/dev/block/bootdevice/by-name
  set_read_write
  BLOCK=/dev/block/mapper
  set_read_write
  blockdev --setrw $MAGISKTMP/block/system_ext
  blockdev --setrw $MAGISKTMP/block/vendor
  blockdev --setrw $MAGISKTMP/block/system_root
  blockdev --setrw $MAGISKTMP/block/system
  mount -o rw,remount $SYSTEMX
  mount -o rw,remount $VENDOR
  mount -o rw,remount $SYSTEM
  mount -o rw,remount /system_ext
  mount -o rw,remount /vendor
  mount -o rw,remount /
else
  if [ "$SYSTEM_ROOT" == true ]; then
    SYSTEM=/system_root
  else
    SYSTEM=/system
  fi
  BLOCK=/dev/block/mapper
  if [ -d $BLOCK ]; then
    umount /system_ext
    umount /vendor
    umount $SYSTEM
    mount -o ro -t auto $BLOCK/system_ext$SLOT /system_ext
    mount -o ro -t auto $BLOCK/vendor$SLOT /vendor
    mount -o ro -t auto $BLOCK/system$SLOT $SYSTEM
    BLOCK=/dev/block/bootdevice/by-name
    set_read_write
    BLOCK=/dev/block/mapper
    set_read_write
    mount -o rw,remount -t auto $BLOCK/system_ext$SLOT /system_ext
    mount -o rw,remount -t auto $BLOCK/vendor$SLOT /vendor
    mount -o rw,remount -t auto $BLOCK/system$SLOT $SYSTEM
  else
    BLOCK=/dev/block/bootdevice/by-name
    set_read_write
    mount -o rw,remount /system_ext
    mount -o rw,remount /vendor
    mount -o rw,remount $SYSTEM
  fi
fi

# patch manifest.xml
CHECK=@1.0::IDms/default
FILE="$MAGISKTMP/mirror/vendor/etc/vintf/manifest.xml
$MAGISKTMP/mirror/system/etc/vintf/manifest.xml
$MAGISKTMP/mirror/system_ext/etc/vintf/manifest.xml
/vendor/etc/vintf/manifest.xml
/system/etc/vintf/manifest.xml
/system/system_ext/etc/vintf/manifest.xml"
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.vendor\]: \[1"; then
  FILE=$MAGISKTMP/mirror/vendor/etc/vintf/manifest.xml
  patch_manifest
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system\]: \[1"; then
  FILE=$MAGISKTMP/mirror/system/etc/vintf/manifest.xml
  patch_manifest
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system_ext\]: \[1"; then
  FILE=$MAGISKTMP/mirror/system_ext/etc/vintf/manifest.xml
  patch_manifest
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.vendor\]: \[1"; then
  FILE=/vendor/etc/vintf/manifest.xml
  patch_manifest
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system\]: \[1"; then
  FILE=/system/etc/vintf/manifest.xml
  patch_manifest
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system_ext\]: \[1"; then
  FILE=/system/system_ext/etc/vintf/manifest.xml
  patch_manifest
fi
if ! getprop | grep -Eq "dolby.skip.check\]: \[1"; then
  check_patch
fi

# patch hwservice contexts
CHECK=u:object_r:hal_dms_hwservice:s0
CHECK2=u:object_r:default_android_hwservice:s0
FILE="$MAGISKTMP/mirror/system_ext/etc/selinux/system_ext_hwservice_contexts
$MAGISKTMP/mirror/vendor/etc/selinux/vendor_hwservice_contexts
$MAGISKTMP/mirror/system/etc/selinux/plat_hwservice_contexts
/system/system_ext/etc/selinux/system_ext_hwservice_contexts
/vendor/etc/selinux/vendor_hwservice_contexts
/system/etc/selinux/plat_hwservice_contexts"
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.vendor\]: \[1"; then
  FILE=$MAGISKTMP/mirror/vendor/etc/selinux/vendor_hwservice_contexts
  patch_hwservice
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system\]: \[1"; then
  FILE=$MAGISKTMP/mirror/system/etc/selinux/plat_hwservice_contexts
  patch_hwservice
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system_ext\]: \[1"; then
  FILE=$MAGISKTMP/mirror/system_ext/etc/selinux/system_ext_hwservice_contexts
  patch_hwservice
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.vendor\]: \[1"; then
  FILE=/vendor/etc/selinux/vendor_hwservice_contexts
  patch_hwservice
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system\]: \[1"; then
  FILE=/system/etc/selinux/plat_hwservice_contexts
  patch_hwservice
fi
if ! grep -Eq "$CHECK" $FILE\
&& ! grep -Eq "$CHECK2" $FILE\
&& ! getprop | grep -Eq "dolby.skip.system_ext\]: \[1"; then
  FILE=/system/system_ext/etc/selinux/system_ext_hwservice_contexts
  patch_hwservice
fi
#if ! getprop | grep -Eq "dolby.skip.check\]: \[1"; then
#  check_patch
#fi

# remount
if [ "$BOOTMODE" == true ]; then
  mount -o ro,remount $VENDOR
  mount -o ro,remount $SYSTEMX
  mount -o ro,remount $SYSTEM
  mount -o ro,remount /vendor
  mount -o ro,remount /system_ext
  mount -o ro,remount /
fi

# store
hide_app() {
  mkdir $MODPATH/system/app/$APP
  touch $MODPATH/system/app/$APP/.replace
  mkdir $MODPATH/system/priv-app/$APP
  touch $MODPATH/system/priv-app/$APP/.replace
  mkdir $MODPATH/system/product/app/$APP
  touch $MODPATH/system/product/app/$APP/.replace
  mkdir $MODPATH/system/product/priv-app/$APP
  touch $MODPATH/system/product/priv-app/$APP/.replace
  mkdir $MODPATH/system/product/preinstall/$APP
  touch $MODPATH/system/product/preinstall/$APP/.replace
}
check_app() {
  FILE=`find /system /product -type f -name $APP.apk`
  if echo $FILE | grep -Eq apk; then
    ui_print "  Checking $APP app..."
    ui_print "  Please wait..."
    if grep -Eq $UUID $FILE; then
      ui_print "  Your $APP app will be hidden"
      hide_app
    fi
  fi
}

# dirac
if getprop | grep -Eq "dolby.disable.dirac\]: \[1"; then
  ui_print "- Activating dirac disabler..."
  sed -i 's/#1//g' $MODPATH/.aml.sh
  UUID=e069d9e0-8329-11df-9168-0002a5d5c51b
  APP=XiaomiParts
  check_app
  APP=ZenfoneParts
  check_app
  APP=ZenParts
  check_app
  APP=GalaxyParts
  check_app
  ui_print " "
fi

# misoundfx
if getprop | grep -Eq "dolby.disable.misoundfx\]: \[1"; then
  ui_print "- Activating misoundfx disabler..."
  sed -i 's/#2//g' $MODPATH/.aml.sh
  UUID=5b8e36a5-144a-4c38-b1d7-0002a5d5c51b
  APP=XiaomiParts
  check_app
  APP=ZenfoneParts
  check_app
  APP=ZenParts
  check_app
  APP=GalaxyParts
  check_app
  ui_print " "
fi

# hide
APP=MotoDolbyV3
hide_app
APP=OPSoundTuner
hide_app

# stream mode
PROP=`getprop stream.mode`
FILE=$MODPATH/.aml.sh
if echo "$PROP" | grep -Eq m; then
  ui_print "- Activating music stream..."
  sed -i 's/#m//g' $FILE
  sed -i 's/musicstream=/musicstream=true/g' $MODPATH/acdb.conf
  ui_print " "
else
  APP=AudioFX
  hide_app
  APP=MusicFX
  hide_app
fi
if echo "$PROP" | grep -Eq r; then
  ui_print "- Activating ring stream..."
  sed -i 's/#r//g' $FILE
  ui_print " "
fi
if echo "$PROP" | grep -Eq a; then
  ui_print "- Activating alarm stream..."
  sed -i 's/#a//g' $FILE
  ui_print " "
fi
if echo "$PROP" | grep -Eq v; then
  ui_print "- Activating voice_call stream..."
  sed -i 's/#v//g' $FILE
  ui_print " "
fi
if echo "$PROP" | grep -Eq n; then
  ui_print "- Activating notification stream..."
  sed -i 's/#n//g' $FILE
  ui_print " "
fi

# settings
FILE=$MODPATH/system/vendor/etc/dolby/dax-default.xml
PROP=`getprop dolby.bass`
if [ "$PROP" -gt 0 ]; then
  ui_print "- Enable bass enhancer for all profiles..."
  sed -i 's/bass-enhancer-enable value="false"/bass-enhancer-enable value="true"/g' $FILE
  ui_print "- Changing bass enhancer boost values to $PROP for all profiles..."
  ROW=`grep bass-enhancer-boost $FILE | sed 's/<bass-enhancer-boost value="0"\/>//p'`
  echo $ROW > $TMPDIR/test
  sed -i 's/<bass-enhancer-boost value="//g' $TMPDIR/test
  sed -i 's/"\/>//g' $TMPDIR/test
  ROW=`cat $TMPDIR/test`
  ui_print "  (Default values: $ROW)"
  for ROWS in $ROW; do
    sed -i "s/bass-enhancer-boost value=\"$ROWS\"/bass-enhancer-boost value=\"$PROP\"/g" $FILE
  done
elif [ "$PROP" = true ]; then
  ui_print "- Enable bass enhancer for all profiles..."
  sed -i 's/bass-enhancer-enable value="false"/bass-enhancer-enable value="true"/g' $FILE
elif [ "$PROP" = false ]; then
  ui_print "- Disable bass enhancer for all profiles..."
  sed -i 's/bass-enhancer-enable value="true"/bass-enhancer-enable value="false"/g' $FILE
fi
if getprop | grep -Eq "dolby.virtualizer\]: \[1"; then
  ui_print "- Enable virtualizer for all profiles..."
  sed -i 's/virtualizer-enable value="false"/virtualizer-enable value="true"/g' $FILE
elif getprop | grep -Eq "dolby.virtualizer\]: \[0"; then
  ui_print "- Disable virtualizer for all profiles..."
  sed -i 's/virtualizer-enable value="true"/virtualizer-enable value="false"/g' $FILE
fi
if getprop | grep -Eq "dolby.volumeleveler\]: \[1"; then
  ui_print "- Using default volume leveler settings"
elif getprop | grep -Eq "dolby.volumeleveler\]: \[2"; then
  ui_print "- Enable volume leveler for all profiles..."
  sed -i 's/volume-leveler-enable value="false"/volume-leveler-enable value="true"/g' $FILE
else
  ui_print "- Disable volume leveler for all profiles..."
  sed -i 's/volume-leveler-enable value="true"/volume-leveler-enable value="false"/g' $FILE
fi
ui_print " "

# permission
ui_print "- Setting permission..."
FILE=$MODPATH/system/vendor/bin/hw/*
chmod 0755 $FILE
FOLDER=`find $MODPATH/system/vendor -type d`
for FOLDERS in $FOLDER; do
  chown 0.2000 $FOLDERS
done
chown 0.2000 $FILE
chcon -R u:object_r:system_lib_file:s0 $MODPATH/system/lib*
chcon -R u:object_r:vendor_file:s0 $MODPATH/system/vendor
chcon -R u:object_r:vendor_configs_file:s0 $MODPATH/system/vendor/etc
chcon u:object_r:hal_dms_default_exec:s0 $FILE
ui_print " "

# vendor_overlay
DIR=/product/vendor_overlay
if [ -d $DIR ]; then
  ui_print "- Fixing $DIR mount..."
  cp -rf $DIR/*/* $MODPATH/system/vendor
  ui_print " "
fi

# uninstaller
FILE=DolbyModuleUninstaller.zip
ui_print "- Flash /sdcard/$FILE"
ui_print "  via recovery if you got bootloop"
cp -f $MODPATH/$FILE /sdcard
rm -f $MODPATH/$FILE
ui_print " "

# store
copy_sepolicy_rules() {
  rm -rf /data/unencrypted/magisk/* /cache/magisk/* /metadata/magisk/* /persist/magisk/* /mnt/vendor/persist/magisk/*
  mkdir /data/unencrypted/magisk /cache/magisk /metadata/magisk /persist/magisk /mnt/vendor/persist/magisk

  local RULESDIR
  local active_dir=$(magisk --path)/.magisk/mirror/sepolicy.rules
  if [ -L $active_dir ]; then
    RULESDIR=$(readlink $active_dir)
  elif [ -d /data/unencrypted ] && ! grep ' /data ' /proc/mounts | grep -qE 'dm-|f2fs'; then
    RULESDIR=/data/unencrypted/magisk
  elif grep -q ' /cache ' /proc/mounts; then
    RULESDIR=/cache/magisk
  elif grep -q ' /metadata ' /proc/mounts; then
    RULESDIR=/metadata/magisk
  elif grep -q ' /persist ' /proc/mounts; then
    RULESDIR=/persist/magisk
  elif grep -q ' /mnt/vendor/persist ' /proc/mounts; then
    RULESDIR=/mnt/vendor/persist/magisk
  else
    return
  fi

  for r in /data/adb/modules*/*/sepolicy.rule; do
    [ -f "$r" ] || continue
    local MODDIR=${r%/*}
    [ -f $MODDIR/disable ] && continue
    [ -f $MODDIR/remove ] && continue
    local MODNAME=${MODDIR##*/}
    mkdir -p $RULESDIR/$MODNAME
    cp -f $r $RULESDIR/$MODNAME/sepolicy.rule
  done
}




